/*
ID: pkpanov1
PROG: floot-it
LANG: C
*/

/*
KEYW:
ALGO:
*/

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <stack>
#include <queue>
#include <map>
#include <set>
#include <algorithm>

using namespace std;

#define MAX_TABLE 350
#define MAX_COLOR 500

typedef struct _colreg
{
    map<int, set<int>*> neighbours;
    int median;
    int color;
    int prev;
    int path_len;
    int generation;
    bool zombie;
} colreg;

typedef struct _colset
{
    set<int> regions;
    int region_count;
    int region_captured;
} colset;

enum _DIRECTION
{
    UP,
    DOWN,
    LEFT,
    RIGHT,
    ALL
};

int n, c;
int table[MAX_TABLE][MAX_TABLE];
int region_map[MAX_TABLE][MAX_TABLE];
bool visited[MAX_TABLE][MAX_TABLE];
struct _colset color_set[MAX_COLOR+1];
struct _colreg region[MAX_TABLE*MAX_TABLE+1];
int region_count, current_region_count;
int dist[MAX_TABLE][MAX_TABLE][5];
vector<int> palette;

int find_median(int *mini_ptr, int *minj_ptr)
{
    int i, j, mini = 0, minj = 0, minm = 99999;

    for (i = 1; i < n; i++)
        for (j = 0; j < n; j++)
        {
            if (table[i-1][j] == table[i][j])
                dist[i][j][UP] = dist[i-1][j][UP];
            else
                dist[i][j][UP] = dist[i-1][j][UP]+1;

            dist[i][j][ALL] = max(dist[i][j][UP], dist[i][j][ALL]);
        }

    for (i = n-1-1; i >= 0; i--)
        for (j = 0; j < n; j++)
        {
            if (table[i+1][j] == table[i][j])
                dist[i][j][DOWN] = dist[i+1][j][DOWN];
            else
                dist[i][j][DOWN] = dist[i+1][j][DOWN]+1;

            dist[i][j][ALL] = max(dist[i][j][DOWN], dist[i][j][ALL]);
        }

    for (i = 0; i < n; i++)
        for (j = 1; j < n; j++)
        {
            if (table[i][j-1] == table[i][j])
                dist[i][j][LEFT] = dist[i][j-1][LEFT];
            else
                dist[i][j][LEFT] = dist[i][j-1][LEFT]+1;

            dist[i][j][ALL] = max(dist[i][j][LEFT], dist[i][j][ALL]);
        }

    for (i = 0; i < n; i++)
        for (j = n-1-1; j >= 0; j--)
        {
            if (table[i][j+1] == table[i][j])
                dist[i][j][RIGHT] = dist[i][j+1][RIGHT];
            else
                dist[i][j][RIGHT] = dist[i][j+1][RIGHT]+1;

            dist[i][j][ALL] = max(dist[i][j][RIGHT], dist[i][j][ALL]);
        }

    for (i = 0; i < n; i++)
        for (j = 0; j < n; j++)
            region[region_map[i][j]].median = max(dist[i][j][ALL], region[region_map[i][j]].median);

    for (i = 0; i < n; i++)
        for (j = 0; j < n; j++)
            if (region[region_map[i][j]].median < minm)
            {
                mini = i;
                minj = j;
                minm = region[region_map[i][j]].median;
            }

    //printf("MEDIAN (%d, %d) = %d\n\n", mini, minj, minm);
    int coeff, max_coeff = 0;

    for (i = 0; i < n; i++)
        for (j = 0; j < n; j++)
            if (region[region_map[i][j]].generation != 1234567)
            {
                coeff = 0;
                region[region_map[i][j]].generation = 1234567;

                if (region[region_map[i][j]].median > minm + (16*minm)/100) continue;

                for (map<int, set<int>*>::iterator iter = region[region_map[i][j]].neighbours.begin();
                    iter != region[region_map[i][j]].neighbours.end();
                    iter++)
                {
                    int s = (*iter).second->size();
                    if ( s > 1 )
                    {
                        coeff += (((1+s)*s)/2);
                    }

                }

                //printf("coeff = %d\n", coeff);

                if (coeff > max_coeff)
                {
                    max_coeff = coeff;
                    mini = i;
                    minj = j;
                }
            }
    
    *mini_ptr = mini;
    *minj_ptr = minj;
    
    return region_map[mini][minj];
}

void fill_map(int x, int y, int r, int c)
{
    if (table[x][y] != c) return;

    if (visited[x][y]) return;

    visited[x][y] = true;
    region_map[x][y] = r;

    if (x > 0)   fill_map(x-1, y, r, c);
    if (x < n-1) fill_map(x+1, y, r, c);
    if (y > 0)   fill_map(x, y-1, r, c);
    if (y < n-1) fill_map(x, y+1, r, c);
}

void fill_region(int x, int y, int r)
{
    if (region_map[x][y] != r)
    {
        if (region[r].neighbours.count(table[x][y]) == 0)
            region[r].neighbours[table[x][y]] = new set<int>; 

        region[r].neighbours[table[x][y]]->insert(region_map[x][y]);
        return;
    }

    if (visited[x][y]) return;

    visited[x][y] = true;

    if (x > 0)   fill_region(x-1, y, r);
    if (x < n-1) fill_region(x+1, y, r);
    if (y > 0)   fill_region(x, y-1, r);
    if (y < n-1) fill_region(x, y+1, r);
}

int get_quo(int ridx)
{
    return 0;
}

inline
void cleanup_region(int ridx)
{
    for (map<int, set<int>*>::iterator iter = region[ridx].neighbours.begin();
                                       iter != region[ridx].neighbours.end();
                                       iter++)
    {
        ((*iter).second)->clear();
        delete ((*iter).second);
    }

    region[ridx].neighbours.clear();
    region[ridx].zombie = true;
}

inline
bool change_color(int ridx, int new_color)
{
    int old_color = region[ridx].color;
    int nidx, gnidx;

    //
    // fail, if color is the same
    //
    if (old_color == new_color)
    {
        return false;
    }

    //
    // Change the color
    //
    region[ridx].color = new_color;

    //
    // Mark this region as 'zombie'
    //
    region[ridx].zombie = true;

    //
    // Remove the current color from the color set
    //
    color_set[old_color].regions.erase(ridx);
    color_set[old_color].region_count--;
    current_region_count--;

    //
    // Add the new color to the color set
    //
    color_set[new_color].regions.insert(ridx);
    color_set[new_color].region_count++;
    current_region_count++;

    if (region[ridx].neighbours.count(new_color) == 0)
    {
        true;
    }

    set<int>* neighbours_with_same_color = region[ridx].neighbours[new_color];
    if (neighbours_with_same_color->empty())
    {
        true;
    }

    for (set<int>::iterator iter1 = neighbours_with_same_color->begin();
                            iter1 != neighbours_with_same_color->end();
                            iter1++)
    {
        nidx = *iter1;

        //
        // Merge all the neighbours with color equal to the new color
        //
        color_set[new_color].regions.erase(nidx);
        color_set[new_color].region_count--;
        current_region_count--;

        region[nidx].zombie = true;

        //
        // Add all the neighbours of the neighbours with color equal to the new color,
        // to the neighbours of the desired region
        //
        for (map<int, set<int>*>::iterator iter2 = region[nidx].neighbours.begin();
                                           iter2 != region[nidx].neighbours.end();
                                           iter2++)
        {
            set<int>* grand_neighbours_with_color_i = (*iter2).second;
            
            for (set<int>::iterator iter3 = grand_neighbours_with_color_i->begin();
                                    iter3 != grand_neighbours_with_color_i->end();
                                    iter3++)
            {
                gnidx = *iter3;

                if (!region[gnidx].zombie)
                {
                    if (region[ridx].neighbours.count(region[gnidx].color) == 0)
                    {
                        region[ridx].neighbours[region[gnidx].color] = new set<int>;
                    }

                    region[ridx].neighbours[region[gnidx].color]->insert(gnidx);
                }
            }
        }

        //
        // Region will be no more active
        //
        cleanup_region(nidx);
    }

    neighbours_with_same_color->clear();

    return true;
}

bool compare(const pair<int,int>& a, const pair<int,int>& b)
{
    return a.second > b.second;
}

bool doit(int starting_region)
{
   int desired_color = 1; 
   int ridx, nidx, color = -1;
   static int gen = 0;
   queue<int> fringe;
   bool found = false;

   fringe.push(starting_region);
   region[starting_region].generation = ++gen;
   region[starting_region].path_len = 0;

   while (!fringe.empty())
   {
        ridx = fringe.front();
        fringe.pop();

        for (map<int, set<int>*>::iterator iter1 = region[ridx].neighbours.begin();
                                           iter1 != region[ridx].neighbours.end();
                                           iter1++)
        {
            set<int>* neighbours_with_color_i = (*iter1).second;

            for (set<int>::iterator iter2 = neighbours_with_color_i->begin();
                                    iter2 != neighbours_with_color_i->end();
                                    iter2++)
                if (!region[*iter2].zombie && region[*iter2].generation != gen)
                {
                    nidx = *iter2;
                    color = region[nidx].color;

                    region[nidx].prev = ridx;
                    region[nidx].generation = gen;
                    region[nidx].path_len = region[ridx].path_len+1;

                    color_set[color].region_captured++;
                    fringe.push(nidx);

                    if (color_set[color].region_count == color_set[color].region_captured)
                    {
                        found = true;
                        goto colorize;
                    }
                }
        }
    }

colorize:

   if (current_region_count == 1) return false;
   if (color == -1) return false;
    

    while (!color_set[color].regions.empty())
    {

        vector< pair<int,int> > v;
        v.reserve(200);

        for (set<int>::iterator iii = color_set[color].regions.begin();
                iii != color_set[color].regions.end();
                iii++)
        {
            int t = *iii;

            while (t != starting_region && !region[t].zombie)
            {
                region[*iii].path_len++;
                t = region[t].prev;
            }

            pair<int,int> p(*iii, region[*iii].path_len);
            v.push_back(p);
        }

        sort(v.begin(), v.end(), compare);

        int t = v[0].first;

        color_set[color].regions.erase(t);

        deque<int> de;

        while (t != starting_region)
        {
            if (!region[t].zombie)
                de.push_front(t);
            t = region[t].prev;
        }

        for (deque<int>::iterator di = de.begin();
                di != de.end();
                di++)
        {
            t = *di;

            if (region[t].color != region[starting_region].color && change_color(starting_region, region[t].color))
                palette.push_back(region[t].color);
        }

        while (!de.empty())
        {
            t = de.front();
            region[t].color = region[starting_region].color;
            de.pop_front();
        }

        if (current_region_count == 1) return false;

    }

    for (int i = 1; i < c; i++)
        color_set[i].region_captured = 0;

    return found;
}

int main()
{
    FILE *fin = fopen("flood-it.in", "r");
    //FILE *fout = fopen("flood-it.out", "w");
    int i, j, ridx, mini, minj, starting_region;

    fscanf(fin, "%d %d\n", &n, &c);

    palette.reserve(n*n); 

    for (i = 0; i < n; i++)
        for (j = 0; j < n; j++)
            fscanf(fin, "%d", &table[i][j]);

    memset(visited, 0, sizeof(visited));
    for (i = 0; i < n; i++)
        for (j = 0; j < n; j++)
            if (!region_map[i][j])
            {
                ridx = ++region_count;
                region[ridx].color = table[i][j];
                color_set[table[i][j]].region_count++;
                color_set[table[i][j]].regions.insert(ridx);
                fill_map(i, j, ridx, table[i][j]);
            }

    memset(visited, 0, sizeof(visited));
    for (i = 0; i < n; i++)
        for (j = 0; j < n; j++)
            if (!visited[i][j])
                fill_region(i, j, region_map[i][j]);

    current_region_count = region_count;

    starting_region = 1;
    mini = minj = 0;
    starting_region = find_median(&mini, &minj);

/*
    for (i = 0; i < n; i++)
    {
        for (j = 0; j < n; j++)
            printf("%d ", table[i][j]);

        printf("\n");
    }
*/
/*
    printf("\n");

    for (i = 0; i < n; i++)
    {
        for (j = 0; j < n; j++)
            printf("%02d ", region_map[i][j]);

        printf("\n");
    }
*/
    //printf("START SOLVING...\n");
    while (doit(starting_region));

    freopen("flood-it.out", "w", stdout);

    printf("%d\n", palette.size());
    printf("%d %d", mini, minj);

    for (vector<int>::iterator iter = palette.begin();
                               iter != palette.end();
                               iter++)
        printf("\n%d", *iter);

    return 0;
}
